from box import Box, BoxList
from restfly import APISession
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import Iterator, convert_keys, keys_exists, snake_to_camel


class LSSConfigControllerAPI(APIEndpoint):
    source_log_map = {
        "app_connector_metrics": "zpn_ast_comprehensive_stats",
        "app_connector_status": "zpn_ast_auth_log",
        "audit_logs": "zpn_audit_log",
        "browser_access": "zpn_http_trans_log",
        "private_svc_edge_status": "zpn_sys_auth_log",
        "user_activity": "zpn_trans_log",
        "user_status": "zpn_auth_log",
        "web_inspection": "zpn_waf_http_exchanges_log",
    }

    def __init__(self, api: APISession):
        super().__init__(api)

        self.v2_url = api.v2_url
        self.v2_admin_url = "https://config.private.zscaler.com/mgmtconfig/v2/admin/lssConfig"

    def _create_policy(self, conditions: list) -> list:
        """
        Creates a dict template for feeding conditions into the ZPA Policies API when adding or updating a policy.

        Args:
            conditions (list): List of condition tuples.

        Returns:
            :obj:`dict`: Dictionary containing the LSS Log Receiver Policy conditions template.

        """

        template = []

        for condition in conditions:
            # Template for SAML Policy Rule objects
            if isinstance(condition, tuple) and len(condition) == 2 and condition[0] == "saml":
                operand = {"operands": [{"objectType": "SAML", "entryValues": []}]}
                for item in condition[1]:
                    entry_values = {
                        "lhs": item[0],
                        "rhs": item[1],
                    }
                    operand["operands"][0]["entryValues"].append(entry_values)
            # Template for client_type Policy Rule objects
            elif condition[0] == "client_type":
                operand = {
                    "operands": [
                        {
                            "objectType": condition[0].upper(),
                            "values": [self.get_client_types()[item] for item in condition[1]],
                        }
                    ]
                }
            # Template for all other object types
            else:
                operand = {
                    "operands": [
                        {
                            "objectType": condition[0].upper(),
                            "values": condition[1],
                        }
                    ]
                }
            template.append(operand)

        return template

    def get_client_types(self) -> Box:
        """
        Returns all available LSS Client Types.

        Client Types are used when creating LSS Receiver configs. ZPA uses an internal code for Client Types, e.g.
        ``zpn_client_type_ip_anchoring`` is the Client Type for a ZIA Service Edge. pyZscaler inverts the key/value so
        that you can perform a lookup using a human-readable name in your code (e.g. ``cloud_connector``).

        Returns:
            :obj:`Box`: Dictionary containing all LSS Client Types with human-readable name as the key.

        Examples:
            Print all LSS Client Types:

            >>> print(zpa.lss.get_client_types())

        """
        # ZPA returns a dictionary of client types but the keys are the internal ZPA codes, which our users probably
        # won't know. This method reverses the dictionary, converts the 'normalised' Client Type name to snake_case
        # before returning it so that a lookup can be easily performed using the Client Type name in plain english.
        #
        # Example before:
        # {'zpn_client_type_exporter': 'Web Browser'}
        # Example after:
        # {'web_browser': 'zpn_client_type_exporter'}

        resp = self._get(f"{self.v2_admin_url}/clientTypes")
        reverse_map = {v.lower().replace(" ", "_"): k for k, v in resp.items()}
        return Box(reverse_map)

    def list_configs(self, **kwargs) -> BoxList:
        """
        Returns all configured LSS receivers.

        Keyword Args:
            **max_items (int):
                The maximum number of items to request before stopping iteration.
            **max_pages (int):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int):
                Specifies the page size. The default size is 20, but the maximum size is 500.
            **search (str, optional):
                The search string used to match against features and fields.

        Returns:
            :obj:`BoxList`: List of all configured LSS receivers.

        Examples:
            Print all configured LSS Receivers.

            >>> for lss_config in zpa.lss.list_configs():
            ...    print(config)
        """
        return BoxList(Iterator(self._api, f"{self.v2_url}/lssConfig", **kwargs))

    def get_config(self, lss_id: str) -> Box:
        """
        Returns information on the specified LSS Receiver config.

        Args:
            lss_id (str):
                The unique identifier for the LSS Receiver config.

        Returns:
            :obj:`Box`: The resource record for the LSS Receiver config.

        Examples:
            Print information on the specified LSS Receiver config.

            >>> print(zpa.lss.get_config('99999'))

        """
        return self._get(f"{self.v2_url}/lssConfig/{lss_id}")

    def get_log_formats(self) -> Box:
        """
        Returns all available pre-configured LSS Log Formats.

        LSS Log Formats are provided as either CSV, JSON or TSV. LSS Log Format values can be used when
        creating or updating LSS Log Receiver configs.

        Returns:
            :obj:`Box`: Dictionary containing pre-configured LSS Log Formats.

        Examples:
            >>> for item in zpa.lss.get_log_formats():
            ...    print(item)

        """
        return self._get(f"{self.v2_admin_url}/logType/formats")

    def get_status_codes(self, log_type: str = "all") -> Box:
        """
        Returns a list of LSS Session Status Codes.

        The LSS Session Status codes are used to filter the messages received by LSS. LSS Session Status Codes can be
        used when adding or updating the filters for an LSS Log Receiver.

        Args:
            log_type (str):
                Filter the LSS Session Status Codes by Log Type, accepted values are:

                - ``all``
                - ``app_connector_status``
                - ``private_svc_edge_status``
                - ``user_activity``
                - ``user_status``

                `Defaults to all.`

        Returns:
            :obj:`Box`: Dictionary containing all LSS Session Status Codes.

        Examples:
            Print all LSS Session Status Codes.

            >>> for item in zpa.lss.get_status_codes():
            ...    print(item)

            Print LSS Session Status Codes for `User Activity` log types.

            >>> for item in zpa.lss.get_status_codes(log_type="user_activity"):
            ...    print(item)

        """
        if log_type == "all":
            return self._get(f"{self.v2_admin_url}/statusCodes")
        elif log_type in ["user_activity", "user_status", "private_svc_edge_status", "app_connector_status"]:
            return self._get(f"{self.v2_admin_url}/statusCodes")[self.source_log_map[log_type]]
        else:
            raise ValueError("Incorrect log_type provided.")

    def add_lss_config(
        self,
        lss_host: str,
        lss_port: str,
        name: str,
        source_log_type: str,
        app_connector_group_ids: list = None,
        enabled: bool = True,
        source_log_format: str = "csv",
        use_tls: bool = False,
        **kwargs,
    ) -> Box:
        """
        Adds a new LSS Receiver Config to ZPA.

        Args:
            app_connector_group_ids (list): A list of unique IDs for the App Connector Groups associated with this
                LSS Config. `Defaults to None.`
            enabled (bool): Enable the LSS Receiver. `Defaults to True`.
            lss_host (str): The IP address of the LSS Receiver.
            lss_port (str): The port number for the LSS Receiver.
            name (str): The name of the LSS Config.
            source_log_format (str):
                The format for the logs. Must be one of the following options:

                - ``csv`` - send logs in CSV format
                - ``json`` - send logs in JSON format
                - ``tsv`` - send logs in TSV format

                `Defaults to csv.`
            source_log_type (str):
                The type of logs that will be sent to the receiver as part of this config. Must be one of the following
                options:

                - ``app_connector_metrics``
                - ``app_connector_status``
                - ``audit_logs``
                - ``browser_access``
                - ``private_svc_edge_status``
                - ``user_activity``
                - ``user_status``
            use_tls (bool):
                Enable to use TLS on the log traffic between LSS components. `Defaults to False.`

        Keyword Args:
            description (str):
                Additional information about the LSS Config.
            filter_status_codes (list):
                A list of Session Status Codes that will be excluded by LSS.
            log_stream_content (str):
                Formatter for the log stream content that will be sent to the LSS Host. Only pass this parameter if you
                intend on using custom log stream content.
            policy_rules (list):
                A list of policy rule tuples. Tuples must follow the convention:

                 (`object_type`, [`object_id`]).

                E.g.

                .. code-block:: python

                    ('app_segment_ids', ['11111', '22222']),
                    ('segment_group_ids', ['88888']),
                    ('idp_ids', ['99999']),
                    ('client_type', ['zia_service_edge'])
                    ('saml', [('33333', 'value')])

        Returns:
            :obj:`Box`: The newly created LSS Config resource record.

        Examples:

            Add an LSS Receiver config that receives App Connector Metrics logs.

            .. code-block:: python

                zpa.lss.add_config(
                    app_connector_group_ids=["app_conn_group_id"],
                    lss_host="192.0.2.100,
                    lss_port="8080",
                    name="app_con_metrics_to_siem",
                    source_log_type="app_connector_metrics")

            Add an LSS Receiver config that receives User Activity logs.

            .. code-block:: python

                zpa.lss.add_config(
                    app_connector_group_ids=["app_conn_group_id"],
                    lss_host="192.0.2.100,
                    lss_port="8080",
                    name="user_activity_to_siem",
                    policy_rules=[
                        ("idp", ["idp_id"]),
                        ("app", ["app_seg_id"]),
                        ("app_group", ["app_seg_group_id"]),
                        ("saml", [("saml_attr_id", "saml_attr_value")]),
                    ],
                    source_log_type="user_activity")

            Add an LSS Receiver config that receives User Status logs.

            .. code-block:: python

                zpa.lss.add_config(
                    app_connector_group_ids=["app_conn_group_id"],
                    lss_host="192.0.2.100,
                    lss_port="8080",
                    name="user_activity_to_siem",
                    policy_rules=[
                        ("idp", ["idp_id"]),
                        ("client_type", ["web_browser", "client_connector"]),
                        ("saml", [("attribute_id", "test3")]),
                    ],
                    source_log_type="user_status")

        """
        source_log_type = self.source_log_map[source_log_type]

        # If the user has supplied custom log stream content formatting then we'll use that. Otherwise map the log
        # type to internal ZPA log codes and get the preformatted log stream content formatting directly from ZPA.
        if kwargs.get("log_stream_content"):
            log_stream_content = kwargs.pop("log_stream_content")
        else:
            log_stream_content = self.get_log_formats()[source_log_type][source_log_format]

        payload = {
            "config": {
                "enabled": enabled,
                "lssHost": lss_host,
                "lssPort": lss_port,
                "name": name,
                "format": log_stream_content,
                "sourceLogType": source_log_type,
                "useTls": use_tls,
            },
            "connectorGroups": [{"id": group_id} for group_id in app_connector_group_ids],
        }

        # Convert tuple list to dict and add to payload
        if kwargs.get("policy_rules"):
            payload["policyRuleResource"] = {
                "conditions": self._create_policy(kwargs.pop("policy_rules")),
                "name": kwargs.get("policy_name", "placeholder"),
            }

        # Add Session Status Codes to filter if provided
        if kwargs.get("filter_status_codes"):
            payload["config"]["filter"] = kwargs.pop("filter_status_codes")

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        # return payload
        return self._post(f"{self.v2_url}/lssConfig", json=payload)

    def update_lss_config(self, lss_config_id: str, **kwargs):
        """
        Update the LSS Receiver Config.

        Args:
            lss_config_id (str): The unique id for the LSS Receiver config.
            **kwargs: Optional keyword args.

        Keyword Args:
            description (str):
                Additional information about the LSS Config.
            enabled (bool):
                Enable the LSS host. Defaults to ``True``.
            filter_status_codes (list):
                A list of Session Status Codes that will be excluded by LSS. If you would like to filter all error codes
                then pass the string "all".
            log_stream_content (str):
                Formatter for the log stream content that will be sent to the LSS Host.
            policy_rules (list):
                A list of policy rule tuples. Tuples must follow the convention:

                 (`object_type`, [`object_id`]).

                E.g.

                .. code-block:: python

                    ('app_segment_ids', ['11111', '22222']),
                    ('segment_group_ids', ['88888']),
                    ('idp_ids', ['99999']),
                    ('client_type', ['zpn_client_type_exporter'])
                    ('saml_attributes', [('33333', 'value')])
            source_log_format (str):
                The format for the logs. Must be one of the following options:

                - ``csv`` - send logs in CSV format
                - ``json`` - send logs in JSON format
                - ``tsv`` - send logs in TSV format
            source_log_type (str):
                The type of logs that will be sent to the receiver as part of this config. Must be one of the following
                options:

                - ``app_connector_metrics``
                - ``app_connector_status``
                - ``audit_logs``
                - ``browser_access``
                - ``private_svc_edge_status``
                - ``user_activity``
                - ``user_status``
            use_tls (bool):
                Enable to use TLS on the log traffic between LSS components. Defaults to ``False``.

        Examples:

            Update an LSS Log Receiver config to change from user activity to user status.

            Note that the ``policy_rules`` will need to be modified to be compatible with the chosen
            ``source_log_type``.

            .. code-block:: python

                zpa.lss.update_config(
                    name="user_status_to_siem",
                    policy_rules=[
                        ("idp", ["idp_id"]),
                        ("client_type", ["machine_tunnel"]),
                        ("saml", [("attribute_id", "11111")]),
                    ],
                    source_log_type="user_status")


        """

        # Set payload to value of existing record
        payload = convert_keys(self.get_config(lss_config_id))

        # If the user has supplied custom log stream content formatting then we'll use that. Otherwise, map the log
        # type to internal ZPA log codes and get the preformatted log stream content formatting directly from ZPA.
        if kwargs.get("log_stream_content"):
            payload["config"]["format"] = kwargs.pop("log_stream_content")
        elif kwargs.get("source_log_type"):
            source_log_type = self.source_log_map[kwargs.pop("source_log_type")]
            payload["config"]["sourceLogType"] = source_log_type
            payload["config"]["format"] = self.get_log_formats()[source_log_type][kwargs.pop("source_log_format", "csv")]

        # Iterate kwargs and update payload for keys that we've renamed.
        for k in list(kwargs):
            if k in ["name", "lss_host", "lss_port", "enabled", "use_tls"]:
                payload["config"][snake_to_camel(k)] = kwargs.pop(k)
            elif k == "filter_status_codes":
                payload["config"]["filter"] = kwargs.pop(k)

        # Convert tuple list to dict and add to payload
        if kwargs.get("policy_rules"):
            if keys_exists(payload, "policyRuleResource", "name"):
                policy_name = payload["policyRuleResource"]["name"]
            else:
                policy_name = "placeholder"
            payload["policyRuleResource"] = {
                "conditions": self._create_policy(kwargs.pop("policy_rules")),
                "name": kwargs.pop("policy_name", policy_name),
            }

        # Add additional provided parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        resp = self._put(f"{self.v2_url}/lssConfig/{lss_config_id}", json=payload).status_code

        if resp == 204:
            return self.get_config(lss_config_id)

    def delete_lss_config(self, lss_id: str) -> int:
        """
        Delete the specified LSS Receiver Config.

        Args:
            lss_id (str): The unique identifier for the LSS Receiver Config to be deleted.

        Returns:
            :obj:`int`:
                The response code for the operation.

        Examples:
            Delete an LSS Receiver config.

            >>> zpa.lss.delete_config('99999')

        """
        return self._delete(f"{self.v2_url}/lssConfig/{lss_id}").status_code



from box import Box, BoxList
from restfly.endpoint import APIEndpoint, APISession

from pyzscaler.utils import Iterator


class SCIMAttributesAPI(APIEndpoint):
    def __init__(self, api: APISession):
        super().__init__(api)
        self.user_config_url = api.user_config_url

    def list_attributes_by_idp(self, idp_id: str, **kwargs) -> BoxList:
        """
        Returns a list of all configured SCIM attributes for the specified IdP.

        Args:
            idp_id (str): The unique id of the IdP to retrieve SCIM attributes for.
            **kwargs: Optional keyword args.

        Keyword Args:
            **max_items (int):
                The maximum number of items to request before stopping iteration.
            **max_pages (int):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int):
                Specifies the page size. The default size is 20, but the maximum size is 500.
            **search (str, optional):
                The search string used to match against features and fields.

        Returns:
            :obj:`BoxList`: A list of all configured SCIM attributes for the specified IdP.

        Examples:
            >>> for scim_attribute in zpa.scim_attributes.list_attributes_by_idp('99999'):
            ...    pprint(scim_attribute)

        """
        return BoxList(Iterator(self._api, f"idp/{idp_id}/scimattribute", **kwargs))

    def get_attribute(self, idp_id: str, attribute_id: str) -> Box:
        """
        Returns information on the specified SCIM attribute.

        Args:
            idp_id (str):
                The unique id of the Idp corresponding to the SCIM attribute.
            attribute_id (str):
                The unique id of the SCIM attribute.

        Returns:
            :obj:`Box`: The resource record for the SCIM attribute.

        Examples:
            >>> pprint(zpa.scim_attributes.get_attribute('99999',
            ...    scim_attribute_id="88888"))

        """

        return self._get(f"idp/{idp_id}/scimattribute/{attribute_id}")

    def get_values(self, idp_id: str, attribute_id: str, **kwargs) -> BoxList:
        """
        Returns information on the specified SCIM attributes.

        Args:
            idp_id (str):
                The unique identifier for the IDP.
            attribute_id (str):
                The unique identifier for the attribute.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            **max_items (int):
                The maximum number of items to request before stopping iteration.
            **max_pages (int):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int):
                Specifies the page size. The default size is 20, but the maximum size is 500.
            **search (str, optional):
                The search string used to match against features and fields.

        Returns:
            :obj:`BoxList`: The resource record for the SCIM attribute values.

        Examples:
            >>> pprint(zpa.scim_attributes.get_values('99999', '88888'))

        """
        return BoxList(
            Iterator(self._api, f"{self.user_config_url}/scimattribute/idpId/{idp_id}/attributeId/{attribute_id}", **kwargs)
        )



import os

from box import Box
from restfly.session import APISession

from pyzscaler import __version__

from .devices import DevicesAPI
from .secrets import SecretsAPI
from .session import AuthenticatedSessionAPI


class ZCC(APISession):
    """
    A Controller to access Endpoints in the Zscaler Mobile Admin Portal API.

    The ZCC object stores the session token and simplifies access to CRUD options within the ZCC Portal.

    Attributes:
        client_id (str): The ZCC Client ID generated from the ZCC Portal.
        client_secret (str): The ZCC Client Secret generated from the ZCC Portal.
        cloud (str): The Zscaler cloud for your tenancy, accepted values are:

            * ``zscaler``
            * ``zscalerone``
            * ``zscalertwo``
            * ``zscalerthree``
            * ``zscloud``
            * ``zscalerbeta``
        company_id (str):
            The ZCC Company ID. There seems to be no easy way to obtain this at present. See the note
            at the top of this page for information on how to retrieve the Company ID.
        override_url (str):
            If supplied, this attribute can be used to override the production URL that is derived
            from supplying the `cloud` attribute. Use this attribute if you have a non-standard tenant URL
            (e.g. internal test instance etc). When using this attribute, there is no need to supply the `cloud`
            attribute. The override URL will be prepended to the API endpoint suffixes. The protocol must be included
            i.e. http:// or https://.

    """

    _vendor = "Zscaler"
    _product = "Zscaler Mobile Admin Portal"
    _backoff = 3
    _build = __version__
    _box = True
    _box_attrs = {"camel_killer_box": True}
    _env_base = "ZCC"
    _env_cloud = "zscaler"
    _url = "https://api-mobile.zscaler.net/papi"

    def __init__(self, **kw):
        self._client_id = kw.get("client_id", os.getenv(f"{self._env_base}_CLIENT_ID"))
        self._client_secret = kw.get("client_secret", os.getenv(f"{self._env_base}_CLIENT_SECRET"))
        self._cloud = kw.get("cloud", os.getenv(f"{self._env_base}_CLOUD"))
        self._url = (
            kw.get("override_url", os.getenv(f"{self._env_base}_OVERRIDE_URL"))
            or f"https://api-mobile.{self._env_cloud}.net/papi"
        )
        self.company_id = kw.get("company_id", os.getenv(f"{self._env_base}_COMPANY_ID"))
        self.conv_box = True
        super(ZCC, self).__init__(**kw)

    def _build_session(self, **kwargs) -> Box:
        """Creates a ZCC API session."""
        super(ZCC, self)._build_session(**kwargs)
        self._auth_token = self.session.create_token(client_id=self._client_id, client_secret=self._client_secret)
        return self._session.headers.update({"auth-token": f"{self._auth_token}"})

    @property
    def devices(self):
        """The interface object for the :ref:`ZCC Devices interface <zcc-devices>`."""
        return DevicesAPI(self)

    @property
    def secrets(self):
        """The interface object for the :ref:`ZCC Secrets interface <zcc-secrets>`."""
        return SecretsAPI(self)

    @property
    def session(self):
        """The interface object for the :ref:`ZCC Authenticated Session interface <zcc-session>`."""
        return AuthenticatedSessionAPI(self)



import json

from box import Box, BoxList
from restfly.endpoint import APIEndpoint


class WebDLP(APIEndpoint):
    def list_rules(self, **kwargs) -> BoxList:
        """
        Returns a list of DLP policy rules, excluding SaaS Security API DLP policy rules.

        Returns:
            :obj:`BoxList`: List of Web DLP items.

        Examples:
            Get a list of all Web DLP Items

            >>> results = zia.web_dlp.list_rules()
            ... for item in results:
            ...    print(item)

        """
        return self._get("webDlpRules")

    def get_rule(self, rule_id: str) -> Box:
        """
        Returns a DLP policy rule, excluding SaaS Security API DLP policy rules.

        Args:
            rule_id (str): The unique id for the Web DLP rule.

        Returns:
            :obj:`Box`: The Web DLP Rule resource record.

        Examples:
            Get information on a Web DLP item by ID

            >>> results = zia.web_dlp.get_rule(rule_id='9999')
            ... print(results)

        """
        return self._get(f"webDlpRules/{rule_id}")

    def list_rules_lite(self) -> BoxList:
        """
        Returns the name and ID for all DLP policy rules, excluding SaaS Security API DLP policy rules.

        Returns:
            :obj:`BoxList`: List of Web DLP name/ids.

        Examples:
            Get Web DLP Lite results

            >>> results = zia.web_dlp.list_rules_lite()
            ... for item in results:
            ...    print(item)

        """
        return self._get("webDlpRules/lite")

    def add_rule(self, payload: json) -> Box:
        """
        Adds a new DLP policy rule.

        Args:
            payload (dict): Dictionary containing the Web DLP Policy rule to be added.

        Returns:
            :obj:`Box`: The newly added Web DLP Policy Rule resource record.

        Payload:
            Minimum items required in payload::

                payload = {
                    'order': 1, # A number greater than 0.
                    'rank': 0,
                    'name': "Dax testing pyZscaler post.",
                    'protocols': ["ANY_RULE"],
                    'action': "ALLOW",
                }

        Examples:
            Add a Web DLP Policy rule with the minimum required parameters::

                payload = {
                    'order': 1,
                    'rank': 0,
                    'name': "Dax testing pyZscaler post.",
                    'protocols': ["ANY_RULE"],
                    'action': "ALLOW",
                }

                 # Add new Web DLP item
                 print(zia.web_dlp.add_rule(payload=payload))

        """
        return self._post("webDlpRules", json=payload)

    def update_rule(self, rule_id: str, payload: dict) -> Box:
        """
        Updates a DLP policy rule. This endpoint is not applicable to SaaS Security API DLP policy rules.

        Args:
            rule_id (str): String of ID.
            payload (dict): Dictionary containing the updated Web DLP Policy Rule.

        Returns:
            :obj:`Box`: The updated Web DLP Policy Rule resource record.

        Examples:
            Update a Web DLP Policy Rule::

                 payload = zia.web_dlp.get_rule('9999')
                 payload['name'] = "daxm updated name."
                 results = zia.web_dlp.update_rule(rule_id=9999, payload=payload)
                 print(results)

        """
        return self._put(f"webDlpRules/{rule_id}", json=payload)

    def delete_rule(self, rule_id: str) -> Box:
        """
        Deletes a DLP policy rule. This endpoint is not applicable to SaaS Security API DLP policy rules.

        Args:
            rule_id (str): Unique id of the Web DLP Policy Rule that will be deleted.

        Returns:
            :obj:`Box`: Response message from the ZIA API endpoint.

        Examples:
            Delete a rule with an id of 9999.

            >>> results = zia.web_dlp.delete_rule(rule_id=9999)
            ... print(results)


        """
        return self._delete(f"webDlpRules/{rule_id}")



import os

from restfly.session import APISession

from pyzscaler import __version__
from pyzscaler.zpa.app_segments import AppSegmentsAPI
from pyzscaler.zpa.certificates import CertificatesAPI
from pyzscaler.zpa.cloud_connector_groups import CloudConnectorGroupsAPI
from pyzscaler.zpa.connector_groups import ConnectorGroupsAPI
from pyzscaler.zpa.connectors import ConnectorsAPI
from pyzscaler.zpa.idp import IDPControllerAPI
from pyzscaler.zpa.inspection import InspectionControllerAPI
from pyzscaler.zpa.lss import LSSConfigControllerAPI
from pyzscaler.zpa.machine_groups import MachineGroupsAPI
from pyzscaler.zpa.policies import PolicySetsAPI
from pyzscaler.zpa.posture_profiles import PostureProfilesAPI
from pyzscaler.zpa.provisioning import ProvisioningAPI
from pyzscaler.zpa.saml_attributes import SAMLAttributesAPI
from pyzscaler.zpa.scim_attributes import SCIMAttributesAPI
from pyzscaler.zpa.scim_groups import SCIMGroupsAPI
from pyzscaler.zpa.segment_groups import SegmentGroupsAPI
from pyzscaler.zpa.server_groups import ServerGroupsAPI
from pyzscaler.zpa.servers import AppServersAPI
from pyzscaler.zpa.service_edges import ServiceEdgesAPI
from pyzscaler.zpa.session import AuthenticatedSessionAPI
from pyzscaler.zpa.trusted_networks import TrustedNetworksAPI


class ZPA(APISession):
    """A Controller to access Endpoints in the Zscaler Private Access (ZPA) API.

    The ZPA object stores the session token and simplifies access to API interfaces within ZPA.

    Attributes:
        client_id (str): The ZPA API client ID generated from the ZPA console.
        client_secret (str): The ZPA API client secret generated from the ZPA console.
        customer_id (str): The ZPA tenant ID found in the Administration > Company menu in the ZPA console.
        cloud (str): The Zscaler cloud for your tenancy, accepted values are:

            * ``production``
            * ``beta``

            Defaults to ``production``.
        override_url (str):
            If supplied, this attribute can be used to override the production URL that is derived
            from supplying the `cloud` attribute. Use this attribute if you have a non-standard tenant URL
            (e.g. internal test instance etc). When using this attribute, there is no need to supply the `cloud`
            attribute. The override URL will be prepended to the API endpoint suffixes. The protocol must be included
            i.e. http:// or https://.

    """

    _vendor = "Zscaler"
    _product = "Zscaler Private Access"
    _build = __version__
    _box = True
    _box_attrs = {"camel_killer_box": True}
    _env_base = "ZPA"
    _url = "https://config.private.zscaler.com"

    def __init__(self, **kw):
        self._client_id = kw.get("client_id", os.getenv(f"{self._env_base}_CLIENT_ID"))
        self._client_secret = kw.get("client_secret", os.getenv(f"{self._env_base}_CLIENT_SECRET"))
        self._customer_id = kw.get("customer_id", os.getenv(f"{self._env_base}_CUSTOMER_ID"))
        self._cloud = kw.get("cloud", os.getenv(f"{self._env_base}_CLOUD"))
        self._override_url = kw.get("override_url", os.getenv(f"{self._env_base}_OVERRIDE_URL"))
        self.conv_box = True
        super(ZPA, self).__init__(**kw)

    def _build_session(self, **kwargs) -> None:
        """Creates a ZPA API authenticated session."""
        super(ZPA, self)._build_session(**kwargs)

        # Configure URL base for this API session
        if self._override_url:
            self._url_base = self._override_url
        elif not self._cloud or self._cloud == "production":
            self._url_base = "https://config.private.zscaler.com"
        elif self._cloud == "beta":
            self._url_base = "https://config.zpabeta.net"
        else:
            raise ValueError("Missing Attribute: You must specify either cloud or override_url")

        # Configure URLs for this API session
        self._url = f"{self._url_base}/mgmtconfig/v1/admin/customers/{self._customer_id}"
        self.user_config_url = f"{self._url_base}/userconfig/v1/customers/{self._customer_id}"
        # The v2 URL supports additional API endpoints
        self.v2_url = f"{self._url_base}/mgmtconfig/v2/admin/customers/{self._customer_id}"

        self._auth_token = self.session.create_token(client_id=self._client_id, client_secret=self._client_secret)
        return self._session.headers.update({"Authorization": f"Bearer {self._auth_token}"})

    @property
    def app_segments(self):
        """
        The interface object for the :ref:`ZPA Application Segments interface <zpa-app_segments>`.

        """
        return AppSegmentsAPI(self)

    @property
    def certificates(self):
        """
        The interface object for the :ref:`ZPA Browser Access Certificates interface <zpa-certificates>`.

        """
        return CertificatesAPI(self)

    @property
    def cloud_connector_groups(self):
        """
        The interface object for the :ref:`ZPA Cloud Connector Groups interface <zpa-cloud_connector_groups>`.

        """
        return CloudConnectorGroupsAPI(self)

    @property
    def connector_groups(self):
        """
        The interface object for the :ref:`ZPA Connector Groups interface <zpa-connector_groups>`.

        """
        return ConnectorGroupsAPI(self)

    @property
    def connectors(self):
        """
        The interface object for the :ref:`ZPA Connectors interface <zpa-connectors>`.

        """
        return ConnectorsAPI(self)

    @property
    def idp(self):
        """
        The interface object for the :ref:`ZPA IDP interface <zpa-idp>`.

        """
        return IDPControllerAPI(self)

    @property
    def inspection(self):
        """
        The interface object for the :ref:`ZPA Inspection interface <zpa-inspection>`.

        """
        return InspectionControllerAPI(self)

    @property
    def lss(self):
        """
        The interface object for the :ref:`ZIA Log Streaming Service Config interface <zpa-lss>`.

        """
        return LSSConfigControllerAPI(self)

    @property
    def machine_groups(self):
        """
        The interface object for the :ref:`ZPA Machine Groups interface <zpa-machine_groups>`.

        """
        return MachineGroupsAPI(self)

    @property
    def policies(self):
        """
        The interface object for the :ref:`ZPA Policy Sets interface <zpa-policies>`.

        """
        return PolicySetsAPI(self)

    @property
    def posture_profiles(self):
        """
        The interface object for the :ref:`ZPA Posture Profiles interface <zpa-posture_profiles>`.

        """
        return PostureProfilesAPI(self)

    @property
    def provisioning(self):
        """
        The interface object for the :ref:`ZPA Provisioning interface <zpa-provisioning>`.

        """
        return ProvisioningAPI(self)

    @property
    def saml_attributes(self):
        """
        The interface object for the :ref:`ZPA SAML Attributes interface <zpa-saml_attributes>`.

        """
        return SAMLAttributesAPI(self)

    @property
    def scim_attributes(self):
        """
        The interface object for the :ref:`ZPA SCIM Attributes interface <zpa-scim_attributes>`.

        """
        return SCIMAttributesAPI(self)

    @property
    def scim_groups(self):
        """
        The interface object for the :ref:`ZPA SCIM Groups interface <zpa-scim_groups>`.

        """
        return SCIMGroupsAPI(self)

    @property
    def segment_groups(self):
        """
        The interface object for the :ref:`ZPA Segment Groups interface <zpa-segment_groups>`.

        """
        return SegmentGroupsAPI(self)

    @property
    def server_groups(self):
        """
        The interface object for the :ref:`ZPA Server Groups interface <zpa-server_groups>`.

        """
        return ServerGroupsAPI(self)

    @property
    def servers(self):
        """
        The interface object for the :ref:`ZPA Application Servers interface <zpa-app_servers>`.

        """
        return AppServersAPI(self)

    @property
    def service_edges(self):
        """
        The interface object for the :ref:`ZPA Service Edges interface <zpa-service_edges>`.

        """
        return ServiceEdgesAPI(self)

    @property
    def session(self):
        """
        The interface object for the :ref:`ZPA Session API calls <zpa-session>`.

        """

        return AuthenticatedSessionAPI(self)

    @property
    def trusted_networks(self):
        """
        The interface object for the :ref:`ZPA Trusted Networks interface <zpa-trusted_networks>`.

        """
        return TrustedNetworksAPI(self)



from box import Box, BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import Iterator, convert_keys, snake_to_camel


class InspectionControllerAPI(APIEndpoint):
    @staticmethod
    def _create_rule(rule: dict) -> dict:
        """
        Creates a rule template for the ZPA Inspection Control API when adding or updating a custom control.

        Args:
            rule (dict): Custom Control rule.

        Returns:
            :obj:`dict`: The Custom Control Rule template.

        """

        rule_set = {
            "names": rule["names"],
            "type": rule["type"],
            "conditions": [],
        }
        for condition in rule["conditions"]:
            rule_set["conditions"].append(
                {
                    "lhs": condition[0],
                    "op": condition[1],
                    "rhs": condition[2],
                }
            )

        return rule_set

    def add_custom_control(
        self,
        name: str,
        default_action: str,
        severity: str,
        type: str,
        rules: list,
        **kwargs,
    ) -> Box:
        """
        Adds a new ZPA Inspection Custom Control.

        Args:
            name (str):
                The name of the custom control.
            default_action (str):
                The default action to take for matches against this custom control. Valid options are:

                - ``PASS``
                - ``BLOCK``
                - ``REDIRECT``
            severity (str):
                The severity for events that match this custom control. Valid options are:

                - ``CRITICAL``
                - ``ERROR``
                - ``WARNING``
                - ``INFO``
            type (str):
                The type of HTTP message this control matches. Valid options are:

                - ``REQUEST``
                - ``RESPONSE``
            rules (list):
                A list of Inspection Control rule objects, with each object using the format::

                    {
                        "names": ["name1", "name2"],
                        "type": "rule_type",
                        "conditions": [
                            ("LHS", "OP", "RHS"),
                            ("LHS", "OP", "RHS"),
                        ],
                    }
            **kwargs: Optional keyword args.

        Keyword Args:
            **description (str):
                Additional information about the custom control.
            **paranoia_level (int):
                The paranoia level for the custom control.

        Returns:
            :obj:`Box`: The newly created custom Inspection Control resource record.

        Examples:
            Create a new custom Inspection Control with the minimum required parameters

            .. code-block:: python

                print(
                    zpa.inspection.add_custom_control(
                        "test8",
                        severity="INFO",
                        description="test descr",
                        paranoia_level="3",
                        type="REQUEST",
                        default_action="BLOCK",
                        rules=[
                            {
                                "names": ["test"],
                                "type": "REQUEST_HEADERS",
                                "conditions": [("SIZE", "GE", "10"), ("VALUE", "CONTAINS", "test")],
                            }
                        ],
                    )
                )

        """

        payload = {
            "name": name,
            "defaultAction": default_action,
            "severity": severity,
            "rules": [],
            "type": type,
        }

        # Use the create_rule method to restructure the Inspection Control rule and add to the payload.
        for rule in rules:
            payload["rules"].append(self._create_rule(rule))

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[key] = value

        # Convert snake to camelcase
        payload = convert_keys(payload)

        return self._post("inspectionControls/custom", json=payload)

    def add_profile(self, name: str, paranoia_level: int, predef_controls_version: str, **kwargs):
        """
        Adds a ZPA Inspection Profile.

        Args:
            name (str):
                The name of the Inspection Profile.
            paranoia_level (int):
                The paranoia level for the Inspection Profile.
            predef_controls_version (str):
                The version of the predefined controls that will be added.
            **kwargs:
                Additional keyword args.

        Keyword Args:
            **description (str):
                Additional information about the Inspection Profile.
            **custom_controls (list):
                A tuple list of custom controls to be added to the Inspection profile.

                Custom control tuples must follow the convention below:

                ``(control_id, action)``

                e.g.

                .. code-block:: python

                    custom_controls = [(99999, "BLOCK"), (88888, "PASS")]
            **predef_controls (list):
                A tuple list of predefined controls to be added to the Inspection profile.

                Predefined control tuples must follow the convention below:

                ``(control_id, action)``

                e.g.

                .. code-block:: python

                    predef_controls = [(77777, "BLOCK"), (66666, "PASS")]

        Returns:
            :obj:`Box`: The newly created Inspection Profile resource record.

        Examples:
            Add a new ZPA Inspection Profile with the minimum required parameters, printing the object to
            console after creation:

            .. code-block:: python

                print(
                    zpa.inspection.add_profile(
                        name="predefined_controls",
                        paranoia_level=3,
                        predef_controls_version="OWASP_CRS/3.3.0",
                    )
                )

            Add a new ZPA Inspection Profile that uses additional predefined controls and custom controls, printing the
            object to console after creation:

            .. code-block:: python

                print(
                    zpa.inspection.add_profile(
                        name="block_common_xss",
                        paranoia_level=2,
                        predefined_controls=[("99999", "BLOCK")],
                        predef_controls_version="OWASP_CRS/3.3.0",
                        custom_controls=[("88888", "BLOCK")],
                    )
                )

        """

        # Inspection Profiles require the default predefined controls to be added. pyZscaler adds these in
        # automatically for our users.

        predef_controls = self.list_predef_controls("OWASP_CRS/3.3.0")
        default_controls = []
        for group in predef_controls:
            if group.default_group:
                default_controls = group.predefined_inspection_controls

        payload = {
            "name": name,
            "paranoiaLevel": paranoia_level,
            "predefinedControls": default_controls,
            "predefinedControlsVersion": predef_controls_version,
        }

        # Extend existing list of default predefined controls if the user supplies more
        if kwargs.get("predef_controls"):
            controls = kwargs.pop("predef_controls")
            for control in controls:
                payload["predefinedControls"].append({"id": control[0], "action": control[1]})

        # Add custom controls if provided
        if kwargs.get("custom_controls"):
            controls = kwargs.pop("custom_controls")
            payload["customControls"] = [{"id": control[0], "action": control[1]} for control in controls]

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[key] = value

        payload = convert_keys(payload)

        return self._post("inspectionProfile", json=payload)

    def delete_custom_control(self, control_id: str) -> int:
        """
        Deletes the specified custom ZPA Inspection Control.

        Args:
            control_id (str):
                The unique id for the custom control that will be deleted.

        Returns:
            :obj:`int`: The status code for the operation.

        Examples:
            Delete a custom ZPA Inspection Control with an id of `99999`.

            .. code-block:: python

                zpa.inspection.delete_custom_control("99999")

        """
        return self._delete(f"inspectionControls/custom/{control_id}").status_code

    def delete_profile(self, profile_id: str):
        """
        Deletes the specified Inspection Profile.

        Args:
            profile_id (str):
                The unique id for the Inspection Profile that will be deleted.

        Returns:
            :obj:`int`: The status code for the operation.

        Examples:
            Delete an Inspection Profile with an id of *999999*:

            .. code-block:: python

                zpa.inspection.delete_profile("999999")

        """
        return self._delete(f"inspectionProfile/{profile_id}").status_code

    def get_custom_control(self, control_id: str) -> Box:
        """
        Returns the specified custom ZPA Inspection Control.

        Args:
            control_id (str):
                The unique id of the custom ZPA Inspection Control to be returned.

        Returns:
            :obj:`Box`: The custom ZPA Inspection Control resource record.

        Examples:
            Print the Custom Inspection Control with an id of `99999`:

            .. code-block:: python

                print(zpa.inspection.get_custom_control("99999"))

        """
        return self._get(f"inspectionControls/custom/{control_id}")

    def get_predef_control(self, control_id: str):
        """
        Returns the specified predefined ZPA Inspection Control.

        Args:
            control_id (str):
                The unique id of the predefined ZPA Inspection Control to be returned.

        Returns:
            :obj:`Box`: The ZPA Inspection Predefined Control resource record.

        Examples:
            Print the ZPA Inspection Predefined Control with an id of `99999`:

            .. code-block:: python

                print(zpa.inspection.get_predef_control("99999"))

        """
        return self._get(f"inspectionControls/predefined/{control_id}")

    def get_profile(self, profile_id: str) -> Box:
        """
        Returns the specified ZPA Inspection Profile.

        Args:
            profile_id (str):
                The unique id of the ZPA Inspection Profile

        Returns:
            :obj:`Box`: The specified ZPA Inspection Profile resource record.

        Examples:
            Print the ZPA Inspection Profile with an id of `99999`:

            .. code-block:: python

                print(zpa.inspection.get_profile("99999"))

        """
        return self._get(f"inspectionProfile/{profile_id}")

    def list_control_action_types(self) -> Box:
        """
        Returns a list of ZPA Inspection Control Action Types.

        Returns:
            :obj:`BoxList`: A list containing the ZPA Inspection Control Action Types.

        Examples:
            Iterate over the ZPA Inspection Control Action Types and print each one:

            .. code-block:: python

                for action_type in zpa.inspection.list_control_action_types():
                    print(action_type)

        """
        return self._get("inspectionControls/actionTypes")

    def list_control_severity_types(self) -> BoxList:
        """
        Returns a list of Inspection Control Severity Types.

        Returns:
            :obj:`BoxList`: A list containing all valid Inspection Control Severity Types.

        Examples:
            Print all Inspection Control Severity Types

            .. code-block:: python

                for severity in zpa.inspection.list_control_severity_types():
                    print(severity)

        """
        return self._get("inspectionControls/severityTypes")

    def list_control_types(self) -> BoxList:
        """
        Returns a list of ZPA Inspection Control Types.

        Returns:
            :obj:`BoxList`: A list containing ZPA Inspection Control Types.

        Examples:
            Print all ZPA Inspection Control Types:

            .. code-block:: python

                for control_type in zpa.inspection.list_control_types():
                    print(control_type)

        """
        return self._get("inspectionControls/controlTypes")

    def list_custom_control_types(self) -> BoxList:
        """
        Returns a list of custom ZPA Inspection Control Types.

        Returns:
            :obj:`BoxList`: A list containing custom ZPA Inspection Control Types.

        Examples:

            Print all custom ZPA Inspection Control Types

            .. code-block:: python

                for control_type in zpa.inspection.list_custom_control_types():
                    print(control_type)

        """
        return self._get("https://config.private.zscaler.com/mgmtconfig/v1/admin/inspectionControls/customControlTypes")

    def list_custom_controls(self, **kwargs) -> BoxList:
        """
        Returns a list of all custom ZPA Inspection Controls.

        Args:
            **kwargs: Optional keyword arguments.

        Keyword Args:
            **search (str):
                The string used to search for a custom control by features and fields.
            **sortdir (str):
                Specifies the sorting order for the search results.

                Accepted values are:

                - ``ASC`` - ascending order
                - ``DESC`` - descending order

        Returns:
            :obj:`BoxList`: A list containing all custom ZPA Inspection Controls.

        Examples:
            Print a list of all custom ZPA Inspection Controls:

            .. code-block:: python

                for control in zpa.inspection.list_custom_controls():
                    print(control)

        """
        return BoxList(Iterator(self._api, "inspectionControls/custom", **kwargs))

    def list_custom_http_methods(self) -> BoxList:
        """
        Returns a list of custom ZPA Inspection Control HTTP Methods.

        Returns:
            :obj:`BoxList`: A list containing custom ZPA Inspection Control HTTP Methods.

        Examples:

            Print all custom ZPA Inspection Control HTTP Methods:

            .. code-block:: python

                for method in zpa.inspection.list_custom_http_methods():
                    print(method)

        """
        return self._get("inspectionControls/custom/httpMethods")

    def list_predef_control_versions(self) -> BoxList:
        """
        Returns a list of predefined ZPA Inspection Control versions.

        Returns:
            :obj:`BoxList`: A list containing all predefined ZPA Inspection Control versions.

        Examples:
            Print all predefined ZPA Inspection Control versions::

                for version in zpa.inspection.list_predef_control_versions():
                    print(version)

        """
        return self._get("inspectionControls/predefined/versions")

    def list_predef_controls(self, version: str, **kwargs) -> BoxList:
        """
        Returns a list of predefined ZPA Inspection Controls.

        Args:
            version (str):
                The version of the predefined controls to return.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            **search (str):
                The string used to search for predefined inspection controls by features and fields.

        Returns:
            :obj:`BoxList`: A list containing all predefined ZPA Inspection Controls that match the Version and Search
            string.

        Examples:
            Return all predefined ZPA Inspection Controls for the given version:

            .. code-block:: python

                for control in zpa.inspection.list_predef_controls(version="OWASP_CRS/3.3.0"):
                    print(control)

            Return predefined ZPA Inspection Controls matching a search string:

            .. code-block:: python

                for control in zpa.inspection.list_predef_controls(search="new_control", version="OWASP_CRS/3.3.0"):
                    print(control)

        """
        payload = {
            "version": version,
        }

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[key] = value

        # Convert snake to camelcase
        payload = convert_keys(payload)

        return self._get("inspectionControls/predefined", params=payload)

    def list_profiles(self, **kwargs) -> BoxList:
        """
        Returns the list of ZPA Inspection Profiles.

        Args:
            **kwargs:
                Optional keyword args.

        Keyword Args:
            **pagesize (int):
                Specifies the page size. The default size is 20 and the maximum size is 500.
            **search (str, optional):
                The search string used to match against features and fields.

        Returns:
            :obj:`BoxList`: The list of ZPA Inspection Profile resource records.

        Examples:
            Iterate over all ZPA Inspection Profiles and print them:

            .. code-block:: python

                for profile in zpa.inspection.list_profiles():
                    print(profile)

        """
        return BoxList(Iterator(self._api, "inspectionProfile", **kwargs))

    def profile_control_attach(self, profile_id: str, action: str, **kwargs) -> Box:
        """
        Attaches or detaches all predefined ZPA Inspection Controls to a ZPA Inspection Profile.

        Args:
            profile_id (str):
                The unique id for the ZPA Inspection Profile that will be modified.
            action (str):
                The association action that will be taken, accepted values are:

                * ``attach``: Attaches all predefined controls to the Inspection Profile with the specified version.
                * ``detach``: Detaches all predefined controls from the Inspection Profile.
            **kwargs:
                Additional keyword arguments.

        Keyword Args:
            profile_version (str):
                The version of the Predefined Controls to attach. Only required when using the
                attach action. Defaults to ``OWASP_CRS/3.3.0``.

        Returns:
            :obj:`Box`: The updated ZPA Inspection Profile resource record.

        Examples:
            Attach all predefined controls to a ZPA Inspection Profile with an id of 99999:

            .. code-block:: python

                updated_profile = zpa.inspection.profile_control_attach("99999", action="attach")

            Attach all predefined controls to a ZPA Inspection Profile with an id of 99999 and specified version:

            .. code-block:: python

                updated_profile = zpa.inspection.profile_control_attach(
                    "99999",
                    action="attach",
                    profile_version="OWASP_CRS/3.2.0",
                )

            Detach all predefined controls from a ZPA Inspection Profile with an id of 99999:

            .. code-block:: python

                updated_profile = zpa.inspection.profile_control_attach(
                    "99999",
                    action="detach",
                )

        Raises:
            ValueError: If an incorrect value is supplied for `action`.

        """
        if action == "attach":
            payload = {"version": kwargs.pop("profile_version", "OWASP_CRS/3.3.0")}
            resp = self._put(f"inspectionProfile/{profile_id}/associateAllPredefinedControls", params=payload)
            return self.get_profile(profile_id) if resp.status_code == 204 else resp.status_code
        elif action == "detach":
            resp = self._put(f"inspectionProfile/{profile_id}/deAssociateAllPredefinedControls")
            return self.get_profile(profile_id) if resp.status_code == 204 else resp.status_code
        else:
            raise ValueError("Unknown action provided. Valid actions are 'attach' or 'detach'.")

    def update_custom_control(self, control_id: str, **kwargs) -> Box:
        """
        Updates the specified custom ZPA Inspection Control.

        Args:
            control_id (str):
                The unique id for the custom control that will be updated.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            **description (str):
                Additional information about the custom control.
            **default_action (str):
                The default action to take for matches against this custom control. Valid options are:

                - ``PASS``
                - ``BLOCK``
                - ``REDIRECT``
            **name (str):
                The name of the custom control.
            **paranoia_level (int):
                The paranoia level for the custom control.
            **rules (list):
                A list of Inspection Control rule objects, with each object using the format::

                    {
                        "names": ["name1", "name2"],
                        "type": "rule_type",
                        "conditions": [
                            ("LHS", "OP", "RHS"),
                            ("LHS", "OP", "RHS"),
                        ],
                    }
            **severity (str):
                The severity for events that match this custom control. Valid options are:

                - ``CRITICAL``
                - ``ERROR``
                - ``WARNING``
                - ``INFO``
            **type (str):
                The type of HTTP message this control matches. Valid options are:

                - ``REQUEST``
                - ``RESPONSE``

        Returns:
            :obj:`Box`: The updated custom ZPA Inspection Control resource record.

        Examples:
            Update the description of a custom ZPA Inspection Control with an id of 99999:

            .. code-block:: python

                print(
                    zpa.inspection.update_custom_control(
                        "99999",
                        description="Updated description",
                    )
                )

            Update the rules of a custom ZPA Inspection Control with an id of 88888:

            .. code-block:: python

                print(
                    zpa.inspection.update_custom_control(
                        "88888",
                        rules=[
                            {
                                "names": ["xforwardedfor_ge_20"],
                                "type": "REQUEST_HEADERS",
                                "conditions": [
                                    ("SIZE", "GE", "20"),
                                    ("VALUE", "CONTAINS", "X-Forwarded-For"),
                                ],
                            }
                        ],
                    )
                )


        """

        # Set payload to value of existing record and recursively convert nested dict keys from snake_case
        # to camelCase.
        payload = convert_keys(self.get_custom_control(control_id))

        # If the user provides rules for an update, clear the current rules then use the create_rule method to
        # restructure the Inspection Control rule and add to the payload.
        if kwargs.get("rules"):
            payload["rules"] = []
            for rule in kwargs.pop("rules"):
                payload["rules"].append(self._create_rule(rule))

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        resp = self._put(f"inspectionControls/custom/{control_id}", json=payload).status_code

        # Return the object if it was updated successfully
        if resp == 204:
            return self.get_custom_control(control_id)

    def update_profile(self, profile_id: str, **kwargs):
        """
        Updates the specified ZPA Inspection Profile.

        Args:
            profile_id (str):
                The unique id for the ZPA Inspection Profile that will be updated.
            predef_controls_version (str):
                The predefined controls version for the ZPA Inspection Profile. Defaults to `OWASP_CRS/3.3.0`.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            **custom_controls (list):
                A tuple list of custom controls to be added to the Inspection profile.

                Custom control tuples must follow the convention below:

                ``(control_id, action)``

                e.g.

                .. code-block:: python

                    custom_controls = [(99999, "BLOCK"), (88888, "PASS")]
            **description (str):
                Additional information about the Inspection Profile.
            **name (str):
                The name of the Inspection Profile.
            **paranoia_level (int):
                The paranoia level for the Inspection Profile.
            **predef_controls (list):
                A tuple list of predefined controls to be added to the Inspection profile.

                Predefined control tuples must follow the convention below:

                ``(control_id, action)``

                e.g.

                .. code-block:: python

                    predef_controls = [(77777, "BLOCK"), (66666, "PASS")]
            **predef_controls_version (str):
                The version of the predefined controls that will be added.

        Returns:
            :obj:`Box`: The updated ZPA Inspection Profile resource record.

        Examples:
            Update the name and description of a ZPA Inspection Profile with the id 99999:

            .. code-block:: python

                print(
                    zpa.inspection.update_profile(
                        "99999",
                        name="inspect_common_predef_controls",
                        description="Inspects common controls from the Predefined set.",
                    )
                )

            Add a custom control to the ZPA Inspection Profile with the id 88888:

            .. code-block:: python

                print(
                    zpa.inspection.update_profile(
                        "88888",
                        custom_controls=[("2", "BLOCK")],
                    )
                )

        """
        # Set payload to value of existing record
        payload = self.get_profile(profile_id)
        payload["predefinedControlsVersion"] = kwargs.get("predef_controls_version", "OWASP_CRS/3.3.0")

        # Extend existing list of default predefined controls if the user supplies more
        if kwargs.get("predef_controls"):
            controls = kwargs.pop("predef_controls")
            for control in controls:
                payload["predefined_controls"] = [{"id": control[0], "action": control[1]} for control in controls]

        # Add custom controls if provided
        if kwargs.get("custom_controls"):
            controls = kwargs.pop("custom_controls")
            payload["custom_controls"] = [{"id": control[0], "action": control[1]} for control in controls]

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[key] = value

        # Convert from snake case to camel case
        payload = convert_keys(payload)

        resp = self._put(f"inspectionProfile/{profile_id}", json=payload).status_code

        # Return the object if it was updated successfully
        if resp == 204:
            return self.get_profile(profile_id)

    def update_profile_and_controls(self, profile_id: str, inspection_profile: dict, **kwargs):
        """
        Updates the inspection profile and controls for the specified ID.

        Note:
            This method has not been fully implemented and will not be maintained. There seems to be functionality
            duplication with the default Inspection Profile update API call. `**kwargs` has been provided as a parameter
            for you to be able to add any additional args that Zscaler may add.

            If you feel that this is in error and that this functionality should be correctly implemented by pyZscaler,
            `raise an issue <https://github.com/mitchos/pyZscaler/issues>`_ in the pyZscaler Github repo.

        Args:
            profile_id (str):
                The unique id of the inspection profile.
            inspection_profile (dict):
                The new inspection profile object.
            **kwargs:
                Additional keyword args.

        """

        payload = {
            "inspection_profile_id": profile_id,
            "inspection_profile": inspection_profile,
        }

        payload = convert_keys(payload)

        return self._patch(f"inspectionProfile/{profile_id}/patch", json=payload).status_code



from box import Box, BoxList
from restfly.endpoint import APIEndpoint, APISession

from pyzscaler.utils import Iterator


class SCIMGroupsAPI(APIEndpoint):
    def __init__(self, api: APISession):
        super().__init__(api)
        self.user_config_url = api.user_config_url

    def list_groups(self, idp_id: str, **kwargs) -> BoxList:
        """
        Returns a list of all configured SCIM groups for the specified IdP.

        Args:
            idp_id (str):
                The unique id of the IdP.

        Keyword Args:
            **end_time (str):
                The end of a time range for requesting last updated data (modified_time) for the SCIM group.
                This requires setting the ``start_time`` parameter as well.
            **idp_group_id (str):
                The unique id of the IdP group.
            **max_items (int):
                The maximum number of items to request before stopping iteration.
            **max_pages (int):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int):
                Specifies the page size. The default size is 20, but the maximum size is 500.
            **scim_user_id (str):
                The unique id for the SCIM user.
            **search (str, optional):
                The search string used to match against features and fields.
            **sort_order (str):
                Sort the last updated time (modified_time) by ascending ``ASC`` or descending ``DSC`` order. Defaults to
                ``DSC``.
            **start_time (str):
                The start of a time range for requesting last updated data (modified_time) for the SCIM group.
                This requires setting the ``end_time`` parameter as well.

        Returns:
            :obj:`list`: A list of all configured SCIM groups.

        Examples:
            >>> for scim_group in zpa.scim_groups.list_groups("999999"):
            ...    pprint(scim_group)

        """
        return BoxList(Iterator(self._api, f"{self.user_config_url}/scimgroup/idpId/{idp_id}", **kwargs))

    def get_group(self, group_id: str, **kwargs) -> Box:
        """
        Returns information on the specified SCIM group.

        Args:
            group_id (str):
                The unique identifier for the SCIM group.
            **kwargs:
                Optional keyword args.

        Keyword Args:
            all_entries (bool):
                Return all SCIM groups including the deleted ones if ``True``. Defaults to ``False``.

        Returns:
            :obj:`dict`: The resource record for the SCIM group.

        Examples:
            >>> pprint(zpa.scim_groups.get_group('99999'))

        """

        return self._get(f"{self.user_config_url}/scimgroup/{group_id}")



from box import Box, BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import Iterator, convert_keys, snake_to_camel


class RuleLabelsAPI(APIEndpoint):
    def list_labels(self, **kwargs) -> BoxList:
        """
        Returns the list of ZIA Rule Labels.

        Keyword Args:
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **page_size (int, optional):
                Specifies the page size. The default size is 100, but the maximum size is 1000.

        Returns:
            :obj:`BoxList`: The list of Rule Labels configured in ZIA.

        Examples:
            List Rule Labels using default settings:

            >>> for label in zia.labels.list_labels():
            ...   print(label)

            List labels, limiting to a maximum of 10 items:

            >>> for label in zia.labels.list_labels(max_items=10):
            ...    print(label)

            List labels, returning 200 items per page for a maximum of 2 pages:

            >>> for label in zia.labels.list_labels(page_size=200, max_pages=2):
            ...    print(label)

        """
        return BoxList(Iterator(self._api, "ruleLabels", **kwargs))

    def get_label(self, label_id: str) -> Box:
        """
        Returns the label details for a given Rule Label.

        Args:
            label_id (str): The unique identifier for the Rule Label.

        Returns:
            :obj:`Box`: The Rule Label resource record.

        Examples:
            >>> label = zia.labels.get_label('99999')

        """
        return self._get(f"ruleLabels/{label_id}")

    def add_label(self, name: str, **kwargs) -> Box:
        """
        Creates a new ZIA Rule Label.

        Args:
            name (str):
                The name of the Rule Label.

        Keyword Args:
            description (str):
                Additional information about the Rule Label.

        Returns:
            :obj:`Box`: The newly added Rule Label resource record.

        Examples:
            Add a label with default parameters:

            >>> label = zia.labels.add_label("My New Label")

            Add a label with description:

            >>> label = zia.labels.add_label("My Second Label":
            ...    description="My second label description")

        """
        payload = {"name": name}

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post("ruleLabels", json=payload)

    def update_label(self, label_id: str, **kwargs):
        """
        Updates information for the specified ZIA Rule Label.

        Args:
            label_id (str): The unique id for the Rule Label that will be updated.

        Keyword Args:
            name (str): The name of the Rule Label.
            description (str): Additional information for the Rule Label.

        Returns:
            :obj:`Box`: The updated Rule Label resource record.

        Examples:
            Update the name of a Rule Label:

            >>> label = zia.labels.update_label(99999,
            ...    name="Updated Label Name")

            Update the name and description of a Rule Label:

            >>> label = zia.labels.update_label(99999,
            ...    name="Updated Label Name",
            ...    description="Updated Label Description")

        """
        # Get the label data from ZIA
        payload = convert_keys(self.get_label(label_id))

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._put(f"ruleLabels/{label_id}", json=payload)

    def delete_label(self, label_id):
        """
        Deletes the specified Rule Label.

        Args:
            label_id (str): The unique identifier of the Rule Label that will be deleted.

        Returns:
            :obj:`int`: The response code for the request.

        Examples
            >>> user = zia.labels.delete_label('99999')

        """

        return self._delete(f"ruleLabels/{label_id}", box=False).status_code



from box import Box, BoxList
from restfly.endpoint import APIEndpoint

from pyzscaler.utils import Iterator, snake_to_camel


def simplify_key_type(key_type):
    # Simplify the key type for our users
    if key_type == "connector":
        return "CONNECTOR_GRP"
    elif key_type == "service_edge":
        return "SERVICE_EDGE_GRP"
    else:
        raise ValueError("Unexpected key type.")


class ProvisioningAPI(APIEndpoint):
    def list_provisioning_keys(self, key_type: str, **kwargs) -> BoxList:
        """
        Returns a list of all configured provisioning keys that match the specified ``key_type``.

        Args:
            key_type (str): The type of provisioning key, accepted values are:

                ``connector`` and ``service_edge``.
            **kwargs: Optional keyword args.

        Keyword Args:
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int, optional):
                Specifies the page size. The default size is 20, but the maximum size is 500.
            **search (str, optional):
                The search string used to match against features and fields.

        Returns:
            :obj:`BoxList`: A list containing the requested provisioning keys.

        Examples:
            List all App Connector provisioning keys.

            >>> for key in zpa.provisioning.list_provisioning_keys(key_type="connector"):
            ...    print(key)

            List all Service Edge provisioning keys.

            >>> for key in zpa.provisioning.list_provisioning_keys(key_type="service_edge"):
            ...    print(key)

        """

        return BoxList(Iterator(self._api, f"associationType/{simplify_key_type(key_type)}/provisioningKey", **kwargs))

    def get_provisioning_key(self, key_id: str, key_type: str) -> Box:
        """
        Returns information on the specified provisioning key.

        Args:
            key_id (str): The unique id of the provisioning key.
            key_type (str): The type of provisioning key, accepted values are:

                ``connector`` and ``service_edge``.

        Returns:
            :obj:`Box`: The requested provisioning key resource record.

        Examples:
            Get the specified App Connector key.

            >>> provisioning_key = zpa.provisioning.get_provisioning_key("999999",
            ...    key_type="connector")

            Get the specified Service Edge key.

            >>> provisioning_key = zpa.provisioning.get_provisioning_key("888888",
            ...    key_type="service_edge")

        """

        return self._get(f"associationType/{simplify_key_type(key_type)}/provisioningKey/{key_id}")

    def add_provisioning_key(
        self,
        key_type: str,
        name: str,
        max_usage: str,
        enrollment_cert_id: str,
        component_id: str,
        **kwargs,
    ) -> Box:
        """
        Adds a new provisioning key to ZPA.

        Args:
            key_type (str): The type of provisioning key, accepted values are:

                ``connector`` and ``service_edge``.
            name (str): The name of the provisioning key.
            max_usage (int): The maximum amount of times this key can be used.
            enrollment_cert_id (str):
                The unique id of the enrollment certificate that will be used for this provisioning key.
            component_id (str):
                The unique id of the component that this provisioning key will be linked to. For App Connectors, this
                will be the App Connector Group Id. For Service Edges, this will be the Service Edge Group Id.
            **kwargs: Optional keyword args.

        Keyword Args:
            enabled (bool): Enable the provisioning key. Defaults to ``True``.

        Returns:
            :obj:`Box`: The newly created Provisioning Key resource record.

        Examples:
            Add a new App Connector Provisioning Key that can be used a maximum of 2 times.

            >>> key = zpa.provisioning.add_provisioning_key(key_type="connector",
            ...    name="Example App Connector Provisioning Key",
            ...    max_usage=2,
            ...    enrollment_cert_id="99999",
            ...    component_id="888888")

            Add a new Service Edge Provisioning Key in the disabled state that can be used once.

            >>> key = zpa.provisioning.add_provisioning_key(key_type="service_edge",
            ...    name="Example Service Edge Provisioning Key",
            ...    max_usage=1,
            ...    enrollment_cert_id="99999",
            ...    component_id="777777"
            ...    enabled=False)

        """

        payload = {
            "name": name,
            "maxUsage": max_usage,
            "enrollmentCertId": enrollment_cert_id,
            "zcomponentId": component_id,
        }

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        return self._post(f"associationType/{simplify_key_type(key_type)}/provisioningKey", json=payload)

    def update_provisioning_key(self, key_id: str, key_type: str, **kwargs) -> Box:
        """
        Updates the specified provisioning key.

        Args:
            key_id (str): The unique id of the Provisioning Key being updated.
            key_type (str): The type of provisioning key, accepted values are:

                ``connector`` and ``service_edge``.
            **kwargs: Optional keyword args.

        Keyword Args:
            name (str): The name of the provisioning key.
            max_usage (int): The maximum amount of times this key can be used.
            enrollment_cert_id (str):
                The unique id of the enrollment certificate that will be used for this provisioning key.
            component_id (str):
                The unique id of the component that this provisioning key will be linked to. For App Connectors, this
                will be the App Connector Group Id. For Service Edges, this will be the Service Edge Group Id.

        Returns:
            :obj:`Box`: The updated Provisioning Key resource record.

        Examples:
            Update the name of an App Connector provisioning key:

            >>> updated_key = zpa.provisioning.update_provisioning_key('999999',
            ...    key_type="connector",
            ...    name="Updated Name")

            Change the max usage of a Service Edge provisioning key:

            >>> updated_key = zpa.provisioning.update_provisioning_key('888888',
            ...    key_type="service_edge",
            ...    max_usage=10)

        """

        # Get the provided provisioning key record
        payload = {snake_to_camel(k): v for k, v in self.get_provisioning_key(key_id, key_type=key_type).items()}

        # Add optional parameters to payload
        for key, value in kwargs.items():
            payload[snake_to_camel(key)] = value

        resp = self._put(f"associationType/{simplify_key_type(key_type)}/provisioningKey/{key_id}", json=payload).status_code

        if resp == 204:
            return self.get_provisioning_key(key_id, key_type=key_type)

    def delete_provisioning_key(self, key_id: str, key_type: str) -> int:
        """
        Deletes the specified provisioning key from ZPA.

        Args:
            key_id (str): The unique id of the provisioning key that will be deleted.
            key_type (str): The type of provisioning key, accepted values are:

                ``connector`` and ``service_edge``.

        Returns:
            :obj:`int`: The status code for the operation.

        Examples:
            Delete an App Connector provisioning key:

            >>> zpa.provisioning.delete_provisioning_key(key_id="999999",
            ...    key_type="connector")

            Delete a Service Edge provisioning key:

            >>> zpa.provisioning.delete_provisioning_key(key_id="888888",
            ...    key_type="service_edge")

        """

        return self._delete(f"associationType/{simplify_key_type(key_type)}/provisioningKey/{key_id}", box=False).status_code



from box import Box, BoxList
from restfly.endpoint import APIEndpoint, APISession

from pyzscaler.utils import Iterator


class CertificatesAPI(APIEndpoint):
    def __init__(self, api: APISession):
        super().__init__(api)

        self.v2_url = api.v2_url

    def list_browser_access(self, **kwargs) -> BoxList:
        """
        Returns a list of all Browser Access certificates.

        Args:
            **kwargs: Optional keyword args.

        Keyword Args:
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int, optional):
                Specifies the page size. The default size is 20, but the maximum size is 500.
            **search (str, optional):
                The search string used to match against features and fields.

        Returns:
            :obj:`BoxList`: List of all Browser Access certificates.

        Examples:
            >>> for cert in zpa.certificates.list_browser_access():
            ...    print(cert)

        """
        return BoxList(Iterator(self._api, f"{self.v2_url}/clientlessCertificate/issued", **kwargs))

    def get_browser_access(self, certificate_id: str) -> Box:
        """
        Returns information on a specified Browser Access certificate.

        Args:
            certificate_id (str):
                The unique identifier for the Browser Access certificate.

        Returns:
            :obj:`Box`:
                The Browser Access certificate resource record.

        Examples:
            >>> ba_certificate = zpa.certificates.get_browser_access('99999')

        """
        return self._get(f"clientlessCertificate/{certificate_id}")

    def get_enrolment(self, certificate_id: str) -> Box:
        """
        Returns information on the specified enrollment certificate.

        Args:
            certificate_id (str): The unique id of the enrollment certificate.

        Returns:
            :obj:`Box`: The enrollment certificate resource record.

        Examples:
            enrolment_cert = zpa.certificates.get_enrolment('99999999')

        """
        return self._get(f"enrollmentCert/{certificate_id}")

    def list_enrolment(self, **kwargs) -> BoxList:
        """
        Returns a list of all configured enrollment certificates.

        Args:
            **kwargs: Optional keyword args.

        Keyword Args:
            **max_items (int, optional):
                The maximum number of items to request before stopping iteration.
            **max_pages (int, optional):
                The maximum number of pages to request before stopping iteration.
            **pagesize (int, optional):
                Specifies the page size. The default size is 20, but the maximum size is 500.
            **search (str, optional):
                The search string used to match against features and fields.

        Returns:
            :obj:`BoxList`: List of all enrollment certificates.

        Examples:
            >>> for cert in zpa.certificates.list_enrolment():
            ...    print(cert)

        """
        return BoxList(Iterator(self._api, f"{self.v2_url}/enrollmentCert", **kwargs))
